from typing import Text

from yaml import ScalarNode, SafeDumper, SafeLoader


class _CustomScalarNode(ScalarNode):

    id = "_custom_scalar"

    def __init__(self, value, tag, style=None):
        super().__init__(value=value, tag=tag, style=style)

    @classmethod
    def from_yaml(cls, loader: SafeLoader, suffix, node: ScalarNode):
        if isinstance(node, ScalarNode):
            return _CustomScalarNode(node.value, suffix, style=node.style)
        else:
            raise NotImplementedError("Node: " + str(type(node)))

    @classmethod
    def to_yaml(cls, dumper: SafeDumper, node: ScalarNode) -> Text:
        return dumper.represent_scalar(node.tag, node.value, style=node.style)



from io import StringIO
from typing import Text

import requests
from yaml import load, SafeLoader, ScalarNode, SafeDumper

from .base import CustomTag


class LinkedTag(CustomTag):

    symbol = "!@"

    def __init__(self, link: Text) -> None:
        self.link = link

    @staticmethod
    def fetch(url: Text) -> StringIO:
        response = requests.get(url)

        if response.ok:
            return StringIO(response.content.decode("utf-8"))
        else:
            raise requests.HTTPError(
                f"Failed to retrieve linked YAML document. "
                f"{response.status_code}: {response.content}"
            )

    @classmethod
    def from_yaml(cls, loader: SafeLoader, node: ScalarNode) -> Text:

        Loader = type(loader)

        if node.value.startswith("http"):
            with cls.fetch(node.value) as target:
                data = load(target, Loader=Loader)

        else:
            with open(node.value) as target:
                data = load(target, Loader=Loader)

        return data

    @classmethod
    def to_yaml(cls, dumper: SafeDumper, data: "LinkedTag") -> ScalarNode:
        return dumper.represent_scalar(cls.symbol, data.link)

    def __repr__(self) -> Text:
        return f"{self.symbol} {self.link}"



"""
LNKD - A simple utility adding custom tags to YAML.
"""

from functools import partial

from .base import CustomTag
from .tag import LinkedTag
from .loader import LinkedLoader
from .dumper import LinkedDumper


__all__ = ["LinkedTag", "CustomTag", "LinkedLoader", "LinkedDumper"]



from typing import TextIO, List
from yaml import SafeLoader, YAMLObject

from .tag import LinkedTag
from .utils import _CustomScalarNode


class LinkedLoader(SafeLoader):

    yaml_constructors = {
        **SafeLoader.yaml_constructors.copy(),
        "!@": LinkedTag.from_yaml,
        "": _CustomScalarNode.from_yaml,
    }
